package com.ibm.xml.dom;

import org.w3c.dom.CharacterData;
import org.w3c.dom.DOMException;
import org.w3c.dom.NodeList;

public abstract class CharacterDataImpl extends NodeImpl implements CharacterData {
   static final long serialVersionUID = 7931170150428474230L;
   private static transient NodeList singletonNodeList;

   protected CharacterDataImpl(DocumentImpl var1, String var2) {
      super(var1, (String)null, var2);
   }

   public abstract String getNodeName();

   public NodeList getChildNodes() {
      if (singletonNodeList == null) {
         singletonNodeList = new 1();
      }

      return singletonNodeList;
   }

   public String getData() {
      if (super.syncData) {
         ((NodeImpl)this).synchronizeData();
      }

      return super.value;
   }

   public int getLength() {
      if (super.syncData) {
         ((NodeImpl)this).synchronizeData();
      }

      return super.value.length();
   }

   public void appendData(String var1) {
      if (super.readOnly) {
         throw new DOMExceptionImpl((short)7, (String)null);
      } else {
         if (super.syncData) {
            ((NodeImpl)this).synchronizeData();
         }

         super.value = super.value + var1;
      }
   }

   public void deleteData(int var1, int var2) throws DOMException {
      if (super.readOnly) {
         throw new DOMExceptionImpl((short)7, (String)null);
      } else if (var2 < 0) {
         throw new DOMExceptionImpl((short)1, (String)null);
      } else {
         if (super.syncData) {
            ((NodeImpl)this).synchronizeData();
         }

         int var3 = Math.max(super.value.length() - var2 - var1, 0);

         try {
            super.value = super.value.substring(0, var1) + (var3 > 0 ? super.value.substring(var1 + var2, var1 + var2 + var3) : "");
         } catch (StringIndexOutOfBoundsException var4) {
            throw new DOMExceptionImpl((short)1, (String)null);
         }
      }
   }

   public void insertData(int var1, String var2) throws DOMException {
      if (super.readOnly) {
         throw new DOMExceptionImpl((short)7, (String)null);
      } else {
         if (super.syncData) {
            ((NodeImpl)this).synchronizeData();
         }

         try {
            super.value = (new StringBuffer(super.value)).insert(var1, var2).toString();
         } catch (StringIndexOutOfBoundsException var3) {
            throw new DOMExceptionImpl((short)1, (String)null);
         }
      }
   }

   public void replaceData(int var1, int var2, String var3) throws DOMException {
      this.deleteData(var1, var2);
      this.insertData(var1, var3);
   }

   public void setData(String var1) throws DOMException {
      ((NodeImpl)this).setNodeValue(var1);
   }

   public String substringData(int var1, int var2) throws DOMException {
      if (super.syncData) {
         ((NodeImpl)this).synchronizeData();
      }

      int var3 = super.value.length();
      if (var2 >= 0 && var1 >= 0 && var1 <= var3 - 1) {
         int var4 = Math.min(var1 + var2, var3);
         return super.value.substring(var1, var4);
      } else {
         throw new DOMExceptionImpl((short)1, (String)null);
      }
   }
}
